#ifndef __COMPAT_NETDEVICE_H__
#   define __COMPAT_NETDEVICE_H__


#include <linux/netdevice.h>

/*
 * The enet_statistics structure moved from linux/if_ether.h to
 * linux/netdevice.h and is renamed net_device_stats in 2.1.25 --hpreg
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 1, 25)
#   include <linux/if_ether.h>

#   define net_device_stats enet_statistics
#endif


/* The netif_rx_ni() API appeared in 2.4.8 --hpreg */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 8)
#   define netif_rx_ni netif_rx
#endif


/* The device struct was renamed net_device in 2.3.14 --hpreg */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 14)
#   define net_device device
#endif


/*
 * SET_MODULE_OWNER appeared sometime during 2.3.x. It was setting
 * dev->owner = THIS_MODULE until 2.5.70, where netdevice refcounting
 * was completely changed.
 *
 * MOD_xxx_USE_COUNT wrappers are here, as they must be mutually
 * exclusive with SET_MODULE_OWNER call.
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
#   define COMPAT_SET_MODULE_OWNER(dev) do {} while (0)
#   define COMPAT_NETDEV_MOD_INC_USE_COUNT MOD_INC_USE_COUNT
#   define COMPAT_NETDEV_MOD_DEC_USE_COUNT MOD_DEC_USE_COUNT
#else
#   define COMPAT_SET_MODULE_OWNER(dev) SET_MODULE_OWNER(dev)
#   define COMPAT_NETDEV_MOD_INC_USE_COUNT do {} while (0)
#   define COMPAT_NETDEV_MOD_DEC_USE_COUNT do {} while (0)
#endif


/*
 * alloc_etherdev and register_netdev must be in sync, either we use init_etherdev
 * and no register_netdev, or we use alloc_etherdev and later we call register_netdev.
 *
 * Prefer alloc_etherdev(), init_etherdev() is dangerous API which calls register_netdev
 * before caller fully initializes device.
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 3)
#define compat_alloc_etherdev(sz)   init_etherdev(NULL, (sz))
#define compat_register_netdev(dev) (0)
#else
#define compat_alloc_etherdev(sz)   alloc_etherdev(sz)
#define compat_register_netdev(dev) register_netdev(dev)
#endif


/* free_netdev() is available since 2.4.23.  Use kfree() on older kernels. */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 23)
#define compat_free_netdev(dev)     kfree(dev)
#else
#define compat_free_netdev(dev)     free_netdev(dev)
#endif


#endif /* __COMPAT_NETDEVICE_H__ */
